/*
  THIS SOURCE CODE IS SUPPLIED ``AS IS'' WITHOUT WARRANTY OF ANY KIND,
  AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
  (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
  WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
  WARRANTIES OR NON INFRINGEMENT. THE USER ASSUMES ALL LIABILITY AND
  RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
  NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR
  DAMAGES OF ANY KIND RESULTING FROM ITS USE. Withot limiting the
  generality of the foregoing, neither the author, nor JAIR, nor JAIR's
  publishers and distributors, warrant that the Source Code will be
  error-free, will operate without interruption, or will meet the needs
  of the user.
*/

/* This program verifies that all maximal tractable subclasses */
/* of the point-interval algebra have been found.              */

/* An algebra is represented as an array of 32 characters,     */
/* where an entry being 1 represents membership,               */
/* and is 0 otherwise.                                         */

#include <stdlib.h>
#include "rcc5.h"          /* Basic definitions      */
#include "rcc5basic.c"     /* Basic primitives       */
#include "rcc5op.c"        /* Operations on algebras */
#include "rcc5io.c"        /* I/O functions          */
#include "rcc5tract.c"     /* Tractability checking  */

void test_alg_of_size(char gen[], char size)

/* This function checks that the algebra "gen" either */
/* is NP-complete, or is included in some algebra     */
/* that is known to be tractable.                     */
/* The algebra is only checked if its closure is of   */
/* the size specified in the "size" argument.         */
/* In order to speed up the process, intractability   */
/* is also checked before the closure                 */
/* computation, but this preserves soundness.         */

{
  char alg[MAXSET];
  int  startsize,i;

  if (!is_intractable(gen) && !is_in_tractable(gen)) {

    for (i=0;i<MAXSET;i++) /* Copy the algebra "gen" to the algebra "alg" */
      alg[i]=gen[i];

    startsize = size;
    compute_closure(alg,&startsize);

    if (!is_intractable(alg)) {
      printf("Test failed:\n");
      printf("Possibly tractable algebra of size %d:\n",size);
      praset(stdout,alg);
    }  

  }
}

void generate_subsets(char from,char size,char gen[],int testing_size)

/* This function recursively generates all subsets of size    */
/* "size" of the point-interval algebra, which contain        */
/* relations with index >= "from", and stores these           */
/* in "gen" (regardless of what has already been stored there.*/
/* The parameter "testing_size" is a parameter that is to be  */
/* passed to the function "test_alg_of_size", when the        */
/* generation reaches zero-size.                              */

{
  if (size==0)
    test_alg_of_size(gen,testing_size);
  else if (from+size<=MAXSET) {
    gen[from]=1;
    generate_subsets(from+1,size-1,gen,testing_size);
    gen[from]=0;
    generate_subsets(from+1,size,gen,testing_size);
  }
}

void init()
{
  init_tractable(); /* Initialise the known-to-be-tractable algebras */
}

void generate_test_subsets()
{
  int i;
  char gen[MAXSET];

  printf("Generating sets of size <= %d...\n",N_TRACTABLE);

  for (i=0;i<=N_TRACTABLE;i++) {
    printf("Size %d\n",i);
    make_null_alg(gen);
    generate_subsets(0,i,gen,i); /* Generate subsets of size i */
  }
  printf("Done\n");
}

void main(int argc, char *argv[])
{
  init();
  generate_test_subsets();
}
