/*
  THIS SOURCE CODE IS SUPPLIED ``AS IS'' WITHOUT WARRANTY OF ANY KIND,
  AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
  (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
  WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
  WARRANTIES OR NON INFRINGEMENT. THE USER ASSUMES ALL LIABILITY AND
  RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
  NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR
  DAMAGES OF ANY KIND RESULTING FROM ITS USE. Withot limiting the
  generality of the foregoing, neither the author, nor JAIR, nor JAIR's
  publishers and distributors, warrant that the Source Code will be
  error-free, will operate without interruption, or will meet the needs
  of the user.
*/

/* This file contains the basic I/O operations needed for reading */
/* and writing point-interval relations and algebras. The basic   */
/* structure is taken from Nebel and Brckert's software for      */
/* investigating the ORD-Horn algebra.                            */

char *sprrel(char *s,char r)
/* Print relation to string */
{

  *s = '\0';
  strcat(s,"( ");
  if (r&RDR)
    strcat(s,"D ");
  if (r&RPO)
    strcat(s,"O ");
  if (r&RPP)
    strcat(s,"< ");
  if (r&RPQ)
    strcat(s,"> ");
  if (r&REQ)
    strcat(s,"= ");
  strcat(s,")");
  return(s);
}

char parserel(char *line)
/* Parse relation from string */
{
  char result;
  char *oldline;
  
  oldline = line;
  result = 0;

  while ((*line != '\0') && (*line != '(')) line++;
  if (*(line++) != '(') {
    fprintf(stderr,"\n*** no '(' in rel spec: '%s'\n",oldline);
    return(-1);
  }
  while (*line == ' ') line++;
  while ((*line != ')' ) && *line) {
    while (*line == ' ') line++;
    switch (*(line++)) {
      case 'D' : result |= RDR; break;
      case 'O' : result |= RPO;  break;
      case '<' : result |= RPP; break;
      case '>' : result |= RPQ; break;
      case '=' : result |= REQ; break;
      default : fprintf(stderr,"\n*** Illegal rel spec: '%s'\n",oldline);
      exit(-1);
    }
    while (*line == ' ')
      line++;
  }
  if (*line != ')') {
    fprintf(stderr,"\n*** Missing ')' in rel spec in '%s'\n",oldline);
    exit(-1);
  }		
  return(result);
}

void praset(FILE *f,char algset[])
/* Print algebra represented as set */
{
  int i;
  char sr[100];

  for (i=0;i<MAXSET;i++)
    if (algset[i])
      fprintf(f,"%s\n",sprrel(sr,i));

}

void striprel(char s[])
{
  int l;

  while (((l=strlen(s))>0) && ((s[l-1]==' ') || (s[l-1]=='\n')))
    s[l-1] = 0;
}

int rdaset(FILE *f,char algset[])
/* Read as set */
{
  char line[100];
  int i;
  int size;

  make_null_alg(algset);  

  size = 0;
  while (fgets(line,100,f)) {

    striprel(line);    
      
    if (line[0] != '\0') {
      i = parserel(line);
      if ((i >= 0) && (algset[i] == 0)) {
        algset[i] = 1;
        size++;
      };
    }
  }
  return(size);
}

int read_alg(char *fname,char a[MAXSET])
/* Read algebra with a given name */
{  

  FILE *algin;

  int algsize;

  printf("Reading algebra %s...\n",fname);

  if ((algin = fopen(fname,"r")) == NULL) {
    fprintf(stderr,"*** Cannot open '%s' for reading\n",fname);
    exit(-1);
  }
  algsize = rdaset(algin,a);
  fclose(algin);

  return(algsize);
}

void add_ext(char *s,char *e)
{
  if (e[0] != '.')
    strcat(s,".");
  strcat(s,e);
}

void print_alg(char *fname,char a[])
{
  FILE *algout;

  if ((algout = fopen(fname,"w")) == NULL) {
    fprintf(stderr,"*** Cannot open '%s' for writing\n",fname);
    exit(-1);
  }
  printf("Writing algebra %s...\n",fname);
  praset(algout,a);
  fclose(algout);
}

