/*
  THIS SOURCE CODE IS SUPPLIED ``AS IS'' WITHOUT WARRANTY OF ANY KIND,
  AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
  (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
  WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
  WARRANTIES OR NON INFRINGEMENT. THE USER ASSUMES ALL LIABILITY AND
  RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
  NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR
  DAMAGES OF ANY KIND RESULTING FROM ITS USE. Withot limiting the
  generality of the foregoing, neither the author, nor JAIR, nor JAIR's
  publishers and distributors, warrant that the Source Code will be
  error-free, will operate without interruption, or will meet the needs
  of the user.
*/

/* This file contains basic operations on the point-interval algebra. */

void make_null_alg(char a[])

/* Set "a" to the empty algebra. */

{
  int i;

  for (i=0;i<MAXSET;i++)
    a[i]=0;
}

char is_subalg(char a1[],char a2[])

/* Check whether "a1" is a subalgebra of "a2" */

{
  int i;

  for (i=0;i<MAXSET;i++)
    if (a1[i] & (!a2[i]))
      return(0);

  return(1);
}

char insert_relation_OK(int  new,
		        char alg[],
		        int  *algsize)

/* Insert a new relation "new" into the algebra "algset",        */
/* increasing "algsize" if "new" was not already in the algebra. */
/* Return 1 if "new" was not included, and 0 otherwise.          */

{	   
  if (!(alg[new])) {
    alg[new]=1;
    (*algsize)++;

    return(1);
  }

  return(0);
}

char is_intractable(char a[])
{
  if (a[RPO] && a[RPP|RPQ])
    return(1);
  if (a[RDR|RPO] && a[RPP|RPQ])
    return(1);

  return(0);
}

void compute_closure(char algset[],
		     int  *algsize)
/* This function computes the closure of the algebra "algset",   */
/* known to be of size "algsize".                                */
/* Both "algset" and "algsize" are modified, to hold the result. */

{
  int r1;
  int r2;
  int oldelecnt;
  
  insert_relation_OK(0,algset,algsize);
  insert_relation_OK(DALL,algset,algsize);

  oldelecnt = 0;
	
  while (oldelecnt != *algsize) {

    oldelecnt = *algsize;
    for (r1=0;r1<MAXSET;r1++){
      if (algset[r1]) 
        insert_relation_OK(inv(r1),algset,algsize);
      for (r2=0;r2<MAXSET;r2++) {
	if (algset[r1] && algset[r2]) {
	  insert_relation_OK(r1&r2,algset,algsize);
	  insert_relation_OK(comp(r1,r2),algset,algsize);

        }
      }
    }
  }
}

